﻿'
' Copyright (c) 2021, MindFusion LLC - Bulgaria.
'

Imports System.Xml


''' <summary>
''' Interaction logic for MainWindow.xaml
''' </summary>
Partial Public Class MainWindow
	Inherits Window

	Public Sub New()
		InitializeComponent()

		terminalBrush = New LinearGradientBrush(Color.FromArgb(&HFF, &HFF, &HFF, &HC0), Color.FromArgb(&HFF, &HBF, &HBF, &H90), 90)
		decisionBrush = New LinearGradientBrush(Color.FromArgb(&HFF, &HF0, &HFF, &HFF), Color.FromArgb(&HFF, &H7F, &HDF, &H8F), 90)
		normalBrush = New LinearGradientBrush(Color.FromArgb(&HFF, &HE0, &HFF, &HFF), Color.FromArgb(&HFF, &HA8, &HBF, &HBF), 90)

		diagram.NodeEffects.Add(New GlassEffect())
	End Sub

	Private Sub loadSample_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
		Dim resourceStream = Application.GetResourceStream(New Uri("flowchart.xml", UriKind.Relative))
		If resourceStream IsNot Nothing AndAlso resourceStream.Stream IsNot Nothing Then
			Using stream = resourceStream.Stream
				Dim doc As New XmlDocument()
				doc.Load(stream)
				diagram.LoadFromXml(doc)
				diagram.NodeEffects.Add(New GlassEffect())
			End Using
		End If
	End Sub

	Private Sub layout_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
		ArrangeDiagram()
	End Sub

	Private Sub layoutRandomDiagram_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
		diagram.ClearAll()

		Dim startNode = diagram.Factory.CreateShapeNode(0, 0, 120, 40)
		startNode.Brush = terminalBrush
		startNode.StrokeThickness = 2
		startNode.FontWeight = FontWeights.Bold
		startNode.Text = "START"
		startNode.Tag = "start"
		startNode.Shape = Shapes.Ellipse

		Dim endNode = diagram.Factory.CreateShapeNode(0, 0, 120, 40)
		endNode.Brush = terminalBrush
		endNode.FontWeight = FontWeights.Bold
		endNode.StrokeThickness = 2
		endNode.Text = "END"
		endNode.Shape = Shapes.Ellipse

		Const maxLength As Integer = 10
		Dim all = New List(Of ShapeNode)()
		Dim remaining = New Queue(Of List(Of ShapeNode))()

		remaining.Enqueue(New List(Of ShapeNode)(New ShapeNode() {startNode}))

		Dim random = New Random(DateTime.Now.Millisecond)
		Dim length As Integer = 0
		While remaining.Count > 0
			Dim newNodes = New List(Of ShapeNode)()
			Dim createdNew As Boolean = False

			Dim [next] = remaining.Dequeue()
			For n As Integer = 0 To [next].Count - 1
				Dim node = [next](n)
				Dim isDecision = TypeOf node.Tag Is Boolean

				Dim previousChoice As ShapeNode = Nothing
				Dim outLinks As Integer = If(isDecision, 2, 1)
				Dim linkTexts As String() = If(isDecision, New String() {"YES", "NO"}, New String() {""})
				For i As Integer = 0 To outLinks - 1
					' Create new or link to an existing node;
					' Create at least one new node for each level
					Dim createNew As Boolean = random.[Next](maxLength) > length
					If Not createdNew AndAlso n = [next].Count - 1 Then
						createNew = True
					End If
					If length = maxLength Then
						createNew = False
					End If

					Dim link As DiagramLink
					If createNew Then
						' New decision or an operation node
						Dim newDecision As Boolean = random.[Next](3) > 0 AndAlso Not isDecision
						Dim newNode = diagram.Factory.CreateShapeNode(0, 0, 120, 40)
						If newDecision Then
							newNode.Shape = Shapes.Decision
							newNode.AnchorPattern = AnchorPattern.Decision2In2Out
							newNode.Brush = decisionBrush
							newNode.Text = "Decision"
							newNode.Tag = True
						Else
							newNode.Shape = Shapes.Rectangle
							newNode.Brush = normalBrush
							newNode.Text = "Operation"
						End If

						link = diagram.Factory.CreateDiagramLink(node, newNode)
						newNodes.Add(newNode)
						createdNew = True
					Else
						' Link to an existing node. If length == maxLength, link to the end node
						If length = maxLength OrElse remaining.Count = 0 Then
							link = diagram.Factory.CreateDiagramLink(node, endNode)
						Else
							' Make sure both choices don't lead to the same node
							Dim choice As ShapeNode = Nothing
							While choice Is previousChoice
								choice = all(random.[Next](all.Count))
							End While
							link = diagram.Factory.CreateDiagramLink(node, choice)
							previousChoice = choice
						End If
					End If

					link.Text = linkTexts(i)
				Next

				all.Add(node)
			Next

			If newNodes.Count > 0 Then
				remaining.Enqueue(newNodes)
			End If

			length += 1
		End While

		ArrangeDiagram()
	End Sub

	Private Sub ArrangeDiagram()
		Dim layout = New MindFusion.Diagramming.Wpf.Layout.DecisionLayout()
		layout.HorizontalPadding = 30
		layout.VerticalPadding = 30
		layout.StartNode = diagram.FindNode("start")
		layout.Arrange(diagram)
	End Sub


	Private terminalBrush As Brush
	Private decisionBrush As Brush
	Private normalBrush As Brush

End Class